// 6620Params.cpp: implementation of the C6620Params class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "6620Params.h"
#include "filters.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_FILESIZE 32000

//#define CREATE_FORMATED_FILE 1

//These tables are used to calculate the decimation stage gains
// which are a function of the decimation rate of each stage
const int CIC2_SCALE_TBL[17] = {
	0,
	0,0,2,2,	//1-16
	3,4,4,4,
	5,5,5,6,
	6,6,6,6
};
const int CIC5_SCALE_TBL[33] = {
	0,
	0,  0, 3, 5,	//1-32
	7,  8,10,10,
	11,12,13,13,
	14,15,15,15,
	16,16,17,17,
	17,18,18,18,
	19,19,19,20,
	20,20,20,20
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

C6620Params::C6620Params()
{
	m_NCOphzDither = TRUE;
	m_NCOampDither = TRUE;
	LoadFixedFilter(5);
}

C6620Params::~C6620Params()
{

}

//////////////////////////////////////////////////////
// Loads in a fixed filter from filters.h
// Indexed from 1 to 12
/////////////////////////////////////////////////////
void C6620Params::LoadFixedFilter(int groupid)
{
INT i;
	switch(groupid)
	{
		case 1:
			m_CIC2Rate = FIL5KHZ_CIC2RATE;
			m_CIC5Rate = FIL5KHZ_CIC5RATE;
			m_RCFRate = FIL5KHZ_RCFRATE;
			m_UseableBW = FIL5KHZ_USEABLEBW;
			for(i=0; i<FIL5KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL5KHZ_COEF[i];
			break;
		case 2:
			m_CIC2Rate = FIL10KHZ_CIC2RATE;
			m_CIC5Rate = FIL10KHZ_CIC5RATE;
			m_RCFRate = FIL10KHZ_RCFRATE;
			m_UseableBW = FIL10KHZ_USEABLEBW;
			for(i=0; i<FIL10KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL10KHZ_COEF[i];
			break;
		case 3:
			m_CIC2Rate = FIL25KHZ_CIC2RATE;
			m_CIC5Rate = FIL25KHZ_CIC5RATE;
			m_RCFRate = FIL25KHZ_RCFRATE;
			m_UseableBW = FIL25KHZ_USEABLEBW;
			for(i=0; i<FIL25KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL25KHZ_COEF[i];
			break;
		case 4:
			m_CIC2Rate = FIL50KHZ_CIC2RATE;
			m_CIC5Rate = FIL50KHZ_CIC5RATE;
			m_RCFRate = FIL50KHZ_RCFRATE;
			m_UseableBW = FIL50KHZ_USEABLEBW;
			for(i=0; i<FIL50KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL50KHZ_COEF[i];
			break;
		case 5:
			m_CIC2Rate = FIL100KHZ_CIC2RATE;
			m_CIC5Rate = FIL100KHZ_CIC5RATE;
			m_RCFRate = FIL100KHZ_RCFRATE;
			m_UseableBW = FIL100KHZ_USEABLEBW;
			for(i=0; i<FIL100KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL100KHZ_COEF[i];
			break;
		case 6:
			m_CIC2Rate = FIL150KHZ_CIC2RATE;
			m_CIC5Rate = FIL150KHZ_CIC5RATE;
			m_RCFRate = FIL150KHZ_RCFRATE;
			m_UseableBW = FIL150KHZ_USEABLEBW;
			for(i=0; i<FIL150KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL150KHZ_COEF[i];
			break;
		case 7:
			m_CIC2Rate = FIL250KHZ_CIC2RATE;
			m_CIC5Rate = FIL250KHZ_CIC5RATE;
			m_RCFRate = FIL250KHZ_RCFRATE;
			m_UseableBW = FIL250KHZ_USEABLEBW;
			for(i=0; i<FIL250KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL250KHZ_COEF[i];
			break;
		case 8:
			m_CIC2Rate = FIL500KHZ_CIC2RATE;
			m_CIC5Rate = FIL500KHZ_CIC5RATE;
			m_RCFRate = FIL500KHZ_RCFRATE;
			m_UseableBW = FIL500KHZ_USEABLEBW;
			for(i=0; i<FIL500KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL500KHZ_COEF[i];
			break;
		case 9:
			m_CIC2Rate = FIL1000KHZ_CIC2RATE;
			m_CIC5Rate = FIL1000KHZ_CIC5RATE;
			m_RCFRate = FIL1000KHZ_RCFRATE;
			m_UseableBW = FIL1000KHZ_USEABLEBW;
			for(i=0; i<FIL1000KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL1000KHZ_COEF[i];
			break;
		case 10:
			m_CIC2Rate = FIL1500KHZ_CIC2RATE;
			m_CIC5Rate = FIL1500KHZ_CIC5RATE;
			m_RCFRate = FIL1500KHZ_RCFRATE;
			m_UseableBW = FIL1500KHZ_USEABLEBW;
			for(i=0; i<FIL1500KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL1500KHZ_COEF[i];
			break;
		case 11:
			m_CIC2Rate = FIL2000KHZ_CIC2RATE;
			m_CIC5Rate = FIL2000KHZ_CIC5RATE;
			m_RCFRate = FIL2000KHZ_RCFRATE;
			m_UseableBW = FIL2000KHZ_USEABLEBW;
			for(i=0; i<FIL2000KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL2000KHZ_COEF[i];
			break;
		case 12:
			m_CIC2Rate = FIL4000KHZ_CIC2RATE;
			m_CIC5Rate = FIL4000KHZ_CIC5RATE;
			m_RCFRate = FIL4000KHZ_RCFRATE;
			m_UseableBW = FIL4000KHZ_USEABLEBW;
			for(i=0; i<FIL4000KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL4000KHZ_COEF[i];
			break;
		case 13:
			m_CIC2Rate = FIL190KHZ_CIC2RATE;
			m_CIC5Rate = FIL190KHZ_CIC5RATE;
			m_RCFRate = FIL190KHZ_RCFRATE;
			m_UseableBW = FIL190KHZ_USEABLEBW;
			for(i=0; i<FIL190KHZ_TAPS; i++)
				m_FIRcoef[i] = FIL190KHZ_COEF[i];
			break;
		default:
			break;
	}
	m_RCFTaps = i;
	m_TotalDecimationRate = m_CIC2Rate*m_CIC5Rate*m_RCFRate;
	m_CIC2Scale = CIC2_SCALE_TBL[m_CIC2Rate];
	m_CIC5Scale = CIC5_SCALE_TBL[m_CIC5Rate];
}

//////////////////////////////////////////////////////
// Loads in a filter from a "SoftCell" format filter file
/////////////////////////////////////////////////////
INT C6620Params::LoadFromFile(CString sPath)
{
CFile cfile;
char* pBuf;
char LineBuf[100];
	if( !cfile.Open( sPath, CFile::modeRead ) )
		return -1;	//return if disk problem or file not found
	int len = cfile.GetLength();
	if( (len <0) || (len>MAX_FILESIZE) )
		return -1;	//return if file too big
	if( len==0 )	//ini file empty
		return 0;
	pBuf = new char[len];	//get copy of file in memory
	if( (int)cfile.Read( pBuf, len) != len )
	{
		delete pBuf;	//if can't read it.
		return -1;
	}
	INT i=0;
	INT j=0;
	j += GetBufLine( pBuf, LineBuf);
	sscanf( LineBuf, "%d", &m_CIC2Rate );
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &m_CIC5Rate );
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &m_RCFRate );
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &i );		//dummy read
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &m_UseableBW );	//read useable BW value
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &i );		//dummy read
	j += GetBufLine( pBuf+j, LineBuf);
	sscanf( LineBuf, "%d", &i );		//dummy read
	int ret;
	i = 0;
	int val;
	do{
		j += GetBufLine( pBuf+j, LineBuf);
		ret = sscanf( LineBuf, "%d", &val );
		m_FIRcoef[i++] = val;
		if(j>=len)
			break;
	}while( ret != EOF || j>=len);
	m_RCFTaps = i;
	m_TotalDecimationRate = m_CIC2Rate*m_CIC5Rate*m_RCFRate;
	m_CIC2Scale = CIC2_SCALE_TBL[m_CIC2Rate];
	m_CIC5Scale = CIC5_SCALE_TBL[m_CIC5Rate];
	delete pBuf;

#ifdef CREATE_FORMATED_FILE
// Some code to dump a filter to a file
CStdioFile cFile;
CString str;
	if( cFile.Open( "c:\\Filter.txt", CFile::modeCreate|CFile::modeWrite) )
	{
		str.Format("{\n");
		cFile.WriteString(str);
		for(i=0; i<m_RCFTaps; i+=1)
		{
			str.Format(" %d,",m_FIRcoef[i]);
			cFile.WriteString(str);
			if( (i&0x000F) == 0x000F )
			{
				str = "\n";
				cFile.WriteString(str);
			}
		}
		str.Format("};");
		cFile.WriteString(str);
	}
	else
		AfxMessageBox("File Error");
#endif
	return 0;
}

// Helper function to get data from a line "SoftCell" format file
int C6620Params::GetBufLine(char *inBuf, char *outBuf)
{
int i =0;
char ch;
	do
	{
		ch = inBuf[i];
		if(ch==0)
			return 0;
		outBuf[i] = inBuf[i++];
	}while( ch>0x0A );
	return i;
}

