// USBio.cpp: implementation of the CUSBio class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdr14x.h"
#include "USBio.h"
#include "sdr14xCtl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// local defines
#define MSGSTATE_HDR1 0		//ASCP partial msg parsing states
#define MSGSTATE_HDR2 1
#define MSGSTATE_DATA 2

#define USB_TIMEOUT 3000	//USB activity timeout

#define MAX_USB_XFER_SIZE (128*64)	//multiple of 64 and pwr of 2 ???  close() hangs if not
#define MAX_USB_BUF_SIZE 65536

#define USB_FIFOSIZE 0x100000		//must be power of 2 for wrap around

#define IOEEROM_SAMPLERATE 0	//External IO EEROM memory address definitions for sdriq
#define NOMINAL_ADFREQ 66666667


#define SDR14_NAME "SDR-14"
#define SDRIQ_NAME "SDR-IQ"


//////////////////////////////////////////////////////////////////////////////
// The following creates a bunch of global pointers to all the FTDI driver
// dll functions
//////////////////////////////////////////////////////////////////////////////
typedef FT_STATUS (WINAPI *PtrToOpen)(PVOID, FT_HANDLE *); 
	static PtrToOpen g_pOpen; 

typedef FT_STATUS (WINAPI *PtrToOpenEx)(PVOID, DWORD, FT_HANDLE *); 
	static PtrToOpenEx g_pOpenEx; 

typedef FT_STATUS (WINAPI *PtrToListDevices)(PVOID, PVOID, DWORD);
	static PtrToListDevices g_pListDevices; 

typedef FT_STATUS (WINAPI *PtrToGetStatus)(FT_HANDLE,LPDWORD,LPDWORD,LPDWORD);
	static PtrToGetStatus g_pGetStatus; 

typedef FT_STATUS (WINAPI *PtrToClose)(FT_HANDLE);
	static PtrToClose g_pClose;

typedef FT_STATUS (WINAPI *PtrToRead)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
	static PtrToRead g_pRead;

typedef FT_STATUS (WINAPI *PtrToWrite)(FT_HANDLE, LPVOID, DWORD, LPDWORD);
	static PtrToWrite g_pWrite;

typedef FT_STATUS (WINAPI *PtrToResetDevice)(FT_HANDLE);
	static PtrToResetDevice g_pResetDevice;
	
typedef FT_STATUS (WINAPI *PtrToPurge)(FT_HANDLE, ULONG);
	static PtrToPurge g_pPurge;
	
typedef FT_STATUS (WINAPI *PtrToSetTimeouts)(FT_HANDLE, ULONG, ULONG);
	static PtrToSetTimeouts g_pSetTimeouts;

typedef FT_STATUS (WINAPI *PtrToGetQueueStatus)(FT_HANDLE, LPDWORD);
	static PtrToGetQueueStatus g_pGetQueueStatus;

typedef FT_STATUS (WINAPI *PtrToSetEventNotification)(FT_HANDLE, DWORD,PVOID);
	static PtrToSetEventNotification g_pSetEventNotification;

typedef FT_STATUS (WINAPI *PtrToSetLatencyTimer)(FT_HANDLE, UCHAR);
	static PtrToSetLatencyTimer g_pSetLatencyTimer;

typedef FT_STATUS (WINAPI *PtrToFT_EE_Read)(FT_HANDLE, PFT_PROGRAM_DATA);
	static PtrToFT_EE_Read g_pFT_EE_Read;

typedef FT_STATUS (WINAPI *PtrFT_SetResetPipeRetryCount)(FT_HANDLE, DWORD);
	static PtrFT_SetResetPipeRetryCount g_pFT_SetResetPipeRetryCount;
	
typedef FT_STATUS (WINAPI *PtrFT_SetUSBParameters)( FT_HANDLE ftHandle, ULONG ulInTransferSize, ULONG ulOutTransferSize);
	static PtrFT_SetUSBParameters g_pFT_SetUSBParameters;
	
typedef FT_STATUS (WINAPI *PtrFT_EE_UARead)( FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen,LPDWORD lpdwBytesRead);
	static PtrFT_EE_UARead g_pFT_EE_UARead;

typedef FT_STATUS (WINAPI *PtrFT_EE_UAWrite)( FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen);
	static PtrFT_EE_UAWrite g_pFT_EE_UAWrite;
	
typedef FT_STATUS (WINAPI *PtrToResetPort)(FT_HANDLE);
	static PtrToResetPort g_pResetPort;
	


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUSBio::CUSBio()
{
	m_USBPortOpen = FALSE;
	m_hReadEvent = NULL;
	m_hmodule = NULL;
	m_Last6620FilterLoaded = -1;
	m_Running = FALSE;
	m_pUSBBuffer = NULL;
	m_hFifoWriteEvent = NULL;
	m_pUSBFifo = NULL;
	m_pUSBFifoThread = NULL;
	InitializeCriticalSection(&m_CriticalSection);
}

CUSBio::~CUSBio()
{
	DeleteResources();
	DeleteCriticalSection(&m_CriticalSection);
}


//////////////////////////////////////////////////////////////////////
//  Called to close USB port
//////////////////////////////////////////////////////////////////////
void CUSBio::CloseUSBPort()
{
	m_Running = FALSE;
	if(m_USBPortOpen  && ( m_FTStatus == FT_OK))
	{
		g_pClose(m_FTHandle);
		m_USBPortOpen = FALSE;
	}
}

//////////////////////////////////////////////////////////////////////
//  Called to open the USB port
// Not all the FTDI dll functions are used.
//////////////////////////////////////////////////////////////////////
BOOL CUSBio::OpenUSBdll()
{
	m_hmodule = LoadLibrary("Ftd2xx.dll");	
	if(m_hmodule == NULL)
		return FALSE;
	g_pWrite = (PtrToWrite)GetProcAddress(m_hmodule, "FT_Write");
	if(g_pWrite == NULL)
		return FALSE;
	g_pRead = (PtrToRead)GetProcAddress(m_hmodule, "FT_Read");
	if(g_pRead == NULL)
		return FALSE;
	g_pOpen = (PtrToOpen)GetProcAddress(m_hmodule, "FT_Open");
	if(g_pOpen == NULL)
		return FALSE;
	g_pOpenEx = (PtrToOpenEx)GetProcAddress(m_hmodule, "FT_OpenEx");
	if(g_pOpenEx == NULL)
		return FALSE;
	g_pListDevices = (PtrToListDevices)GetProcAddress(m_hmodule, "FT_ListDevices");
	if(g_pListDevices == NULL)
		return FALSE;
	g_pClose = (PtrToClose)GetProcAddress(m_hmodule, "FT_Close");
	if(g_pClose == NULL)
		return FALSE;
	g_pResetDevice = (PtrToResetDevice)GetProcAddress(m_hmodule, "FT_ResetDevice");
	if(g_pResetDevice == NULL)
		return FALSE;
	g_pPurge = (PtrToPurge)GetProcAddress(m_hmodule, "FT_Purge");
	if(g_pPurge == NULL)
		return FALSE;
	g_pSetTimeouts = (PtrToSetTimeouts)GetProcAddress(m_hmodule, "FT_SetTimeouts");
	if(g_pSetTimeouts == NULL)
		return FALSE;
	g_pGetQueueStatus = (PtrToGetQueueStatus)GetProcAddress(m_hmodule, "FT_GetQueueStatus");
	if(g_pGetQueueStatus == NULL)
		return FALSE;
	g_pGetStatus = (PtrToGetStatus)GetProcAddress(m_hmodule, "FT_GetStatus");
	if(g_pGetStatus == NULL)
		return FALSE;
	g_pSetEventNotification = (PtrToSetEventNotification)GetProcAddress(m_hmodule, "FT_SetEventNotification");
	if(g_pSetEventNotification == NULL)
		return FALSE;
	g_pSetLatencyTimer = (PtrToSetLatencyTimer)GetProcAddress(m_hmodule, "FT_SetLatencyTimer");
	if(g_pSetLatencyTimer == NULL)
		return FALSE;
	g_pFT_EE_Read = (PtrToFT_EE_Read)GetProcAddress(m_hmodule, "FT_EE_Read");
	if(g_pFT_EE_Read == NULL)
		return FALSE;
	g_pFT_SetResetPipeRetryCount = (PtrFT_SetResetPipeRetryCount)GetProcAddress(m_hmodule, "FT_SetResetPipeRetryCount");
	if(g_pFT_SetResetPipeRetryCount == NULL)
		return FALSE;
	g_pFT_SetUSBParameters = (PtrFT_SetUSBParameters)GetProcAddress(m_hmodule, "FT_SetUSBParameters");
	if(g_pFT_SetUSBParameters == NULL)
		return FALSE;
	g_pFT_EE_UARead = (PtrFT_EE_UARead)GetProcAddress(m_hmodule, "FT_EE_UARead");
	if(g_pFT_EE_UARead == NULL)
		return FALSE;
	g_pFT_EE_UAWrite = (PtrFT_EE_UAWrite)GetProcAddress(m_hmodule, "FT_EE_UAWrite");
	if(g_pFT_EE_UAWrite == NULL)
		return FALSE;
	g_pResetPort = (PtrToResetDevice)GetProcAddress(m_hmodule, "FT_ResetPort");
	if(g_pResetPort == NULL)
		return FALSE;
	return TRUE;

}

///////////////////////////////////////////////////////////////////////////
//   Called to delete any allocated resources in this class
///////////////////////////////////////////////////////////////////////////
void CUSBio::DeleteResources()
{
	CloseUSBPort();
	Sleep(50);
	if(m_hReadEvent)
	{
		CloseHandle(m_hReadEvent);
		m_hReadEvent = NULL;
	}
	if(m_hFifoWriteEvent)
	{
		CloseHandle(m_hFifoWriteEvent);
		m_hFifoWriteEvent = NULL;
	}
	if(m_hmodule)
	{
		FreeLibrary(m_hmodule);
		m_hmodule = NULL;
	}
	if(m_pUSBBuffer)
	{
		delete m_pUSBBuffer;
		m_pUSBBuffer = NULL;
	}
	if(m_pUSBFifo)
	{
		delete m_pUSBFifo;
		m_pUSBFifo = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop of this IO object(run in worker thread)
// This is a worker thread that services the USB port.
//////////////////////////////////////////////////////////////////////
void CUSBio::USBthreadProc(LPVOID pCtrl)
{
DWORD eventstat;
DWORD RxBytes;
DWORD TxBytes;
DWORD bytesGot;
int WaitTimer;
unsigned int IQSampleRate=0;
	CSDR14XCtrl* pXsdrCtrl = (CSDR14XCtrl*)pCtrl;

	OpenUSBdll();	//open the FTDI driver dll
	m_hReadEvent = CreateEvent( NULL, FALSE, FALSE, NULL );
	m_hFifoWriteEvent = CreateEvent(NULL, FALSE,FALSE, NULL);
	m_pUSBBuffer = new unsigned __int8[MAX_USB_BUF_SIZE];
	m_pUSBFifo = new unsigned __int8[USB_FIFOSIZE];
	m_FifoHead = 0;
	m_FifoTail = 0;
	Sleep(100);
	while(pXsdrCtrl->m_ThreadRunning)
	{
		if(!m_USBPortOpen )	//open USB port if not open already
		{
			if(m_hmodule)
			{
				m_FTStatus = g_pOpenEx(SDR14_NAME, FT_OPEN_BY_DESCRIPTION, &m_FTHandle);
				if( m_FTStatus != FT_OK)
				{
					m_FTStatus = g_pOpenEx(SDRIQ_NAME, FT_OPEN_BY_DESCRIPTION, &m_FTHandle);
					if( m_FTStatus == FT_OK)
					{
						AfxGetThread( )->SetThreadPriority(THREAD_PRIORITY_TIME_CRITICAL );
						g_pSetLatencyTimer(m_FTHandle,2);
						pXsdrCtrl->m_SDRtype = SDRTYPE_IQ;
						Sleep(100);
						g_pFT_EE_UARead(m_FTHandle, m_EEROMData, EEROM_SIZE, &bytesGot);	//read sdriq eerom params
						ReadIOMemory( IOEEROM_SAMPLERATE, &IQSampleRate);	//get sample rate from EEROM data
						SetSDRIQSampleRateFromEEROM(IQSampleRate);			//send it back to SDR-IQ
						Sleep(500);
					}
				}
				else
				{
					pXsdrCtrl->m_SDRtype = SDRTYPE_14;
					AfxGetThread( )->SetThreadPriority(THREAD_PRIORITY_TIME_CRITICAL );
				}
			}
			else
			{
				AfxGetThread( )->SetThreadPriority(THREAD_PRIORITY_NORMAL );
				m_FTStatus = FT_DEVICE_NOT_FOUND;
			}
			if( m_FTStatus != FT_OK)
			{	//if error so must not be connected
				AfxGetThread( )->SetThreadPriority(THREAD_PRIORITY_NORMAL );
				m_Running = FALSE;
				::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_NOTCONNECTED, 0);
				WaitTimer = 10;
				while( (--WaitTimer>0) && pXsdrCtrl->m_ThreadRunning)
					Sleep(100);		//wait 1 second before trying again
			}
			else
			{		//Here if SDR-xx was found and USB port opened OK
				m_USBPortOpen = TRUE;
				m_Last6620FilterLoaded = -1;
				m_MsgState = MSGSTATE_HDR1;
				m_RxMsgIndex = 0;
				m_RxMsgLength = 0;
				m_FTStatus = g_pResetDevice(m_FTHandle);
				Sleep(10);
				m_FTStatus = g_pPurge(m_FTHandle,FT_PURGE_RX|FT_PURGE_TX);
				Sleep(10);
				m_FTStatus = g_pSetTimeouts(m_FTHandle, 1500, 1500);
				Sleep(10);
				g_pFT_SetUSBParameters(m_FTHandle, MAX_USB_XFER_SIZE, 0);
				//flush out anything the SDR-XX may have buffered up
				pXsdrCtrl->ErrorStop();	//maKe sure sdrXX is not running
				m_FTStatus = g_pGetQueueStatus(m_FTHandle,&RxBytes);
				if(RxBytes>0)
					m_FTStatus = g_pRead(m_FTHandle, m_pUSBBuffer,	RxBytes, &bytesGot);
				m_Running = FALSE;
				//setup FTDI read event
				m_FTStatus = g_pSetEventNotification(m_FTHandle,FT_EVENT_RXCHAR,m_hReadEvent);
				RxBytes=0;
				// request initial info from SDR-XX.  Need to do only once after connection
				Sleep(500);
				m_txASCPmsg.words.header = CI_GENERAL_HARDFIRM_VERSION_REQLEN;
				m_txASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
				m_txASCPmsg.words.CItem = CI_GENERAL_HARDFIRM_VERSION;
				m_txASCPmsg.bytes.parms[0] = 1;	//req firmware revision
				SendASCPMsg( &m_txASCPmsg);
				m_txASCPmsg.bytes.parms[0] = 0;	//req boot revision
				SendASCPMsg( &m_txASCPmsg);
				m_txASCPmsg.words.header = CI_GENERAL_INTERFACE_NAME_REQLEN;
				m_txASCPmsg.bytes.b1 |= TYPE_HOST_REQ_CITEM;
				m_txASCPmsg.words.CItem = CI_GENERAL_INTERFACE_NAME;
				SendASCPMsg( &m_txASCPmsg);
				m_txASCPmsg.words.CItem = CI_GENERAL_INTERFACE_SERIALNUM;
				SendASCPMsg( &m_txASCPmsg);
				m_txASCPmsg.words.CItem = CI_GENERAL_INTERFACE_VERSION;
				SendASCPMsg( &m_txASCPmsg);
				m_FifoHead = 0;
				m_FifoTail = 0;
			}
		}
		else	//Here with USB port open and ready for action
		{
			//if no data to read just wait for USB read or an event timeout
			eventstat = WaitForSingleObject( m_hReadEvent, USB_TIMEOUT);
			if(eventstat == WAIT_TIMEOUT)
			{	//a timeout and no data is ready so reset msg states
TRACE("Timeout\n\r");
				m_FifoHead = 0;
				m_FifoTail = 0;
				m_RxMsgIndex = 0;		//reset msg decode state pointers
				m_RxMsgLength = 0;
				m_MsgState = MSGSTATE_HDR1;
				m_FTStatus = g_pGetQueueStatus(m_FTHandle, &RxBytes);
				if( m_FTStatus != FT_OK)
				{	//if error probably disconnected
					m_FTStatus = g_pClose(m_FTHandle);
					m_USBPortOpen = FALSE;
					::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_NOTCONNECTED, 0);
				}
				else
				{
					if(m_Running)	//send appropriate status message
						::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_RUNNING, 0);
					else
						::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
				}
			}
			else
			{	//Get Status to find number of RxBytes received
				// ONLY this g_pGetStatus() variant will actually clear the event 
				m_FTStatus = g_pGetStatus(m_FTHandle, &RxBytes, &TxBytes, &eventstat);
				if( m_FTStatus != FT_OK)
				{	//if error probably disconnected
					m_FTStatus = g_pClose(m_FTHandle);
					m_USBPortOpen = FALSE;
					::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_NOTCONNECTED, 0);
				}
				else	//here if event triggered or timeout
				{
					if(RxBytes>0)	//There is data in USB RX queue so read and processs it
					{
						m_FTStatus = g_pRead(m_FTHandle, m_pUSBBuffer,	RxBytes, &bytesGot); //read data
						if( (m_FTStatus != FT_OK) )
						{	//if error, bail out
							pXsdrCtrl->ErrorStop();
							m_FTStatus = g_pClose(m_FTHandle);
							m_USBPortOpen = FALSE;
							::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_NOTCONNECTED, 0);
						}
						else
						{	// process the bytes received so far.
							for(unsigned int i=0; i<bytesGot; i++) //Place in FIFO
							{
								m_pUSBFifo[m_FifoHead++] = m_pUSBBuffer[i];
								m_FifoHead &= (USB_FIFOSIZE-1);
							}
							::SetEvent(m_hFifoWriteEvent);
						}
					}
					else	
					{	//must be some sort of timeout with nothing in USB Rx Queue
						Sleep(0);
					}
				}
			}
		}
	}
	SetEvent(m_hFifoWriteEvent);
	::PostMessage( pXsdrCtrl->m_hWnd, MSG_STATUSCHNG, STATUS_NOTCONNECTED, 0);
	DeleteResources();
}


//////////////////////////////////////////////////////////////////////
//  Called send an ASCP formated message to the SDR-14 using the USB port
//////////////////////////////////////////////////////////////////////
void CUSBio::SendASCPMsg( ASCPstruct* pASCPmsg)
{
	EnterCriticalSection(&m_CriticalSection);
DWORD txbytes;
int timeout = 100;	//xmit timeout ~1 second
DWORD bytesremaining = pASCPmsg->words.header&LENGTH_MASK;
	if(m_USBPortOpen )
	{
		while(bytesremaining > 0)
		{
			m_FTStatus = g_pWrite( m_FTHandle, (LPVOID)pASCPmsg, bytesremaining, &txbytes);
			bytesremaining -= txbytes;
			if(bytesremaining>0)	//if driver didn't send them all
			{
				Sleep(10);			//wait
				if(timeout-- <= 0)	//bail if taking too long
					m_FTStatus = FT_FAILED_TO_WRITE_DEVICE;
			}
			if( m_FTStatus != FT_OK)
			{
				m_FTStatus = g_pClose(m_FTHandle);
				m_USBPortOpen = FALSE;
				bytesremaining = 0;	//force exit of loop
			}
		}
	}
	LeaveCriticalSection(&m_CriticalSection);
}

//////////////////////////////////////////////////////////////////////
//  Called to load the AD6620 parameters of the SDR-14.
//////////////////////////////////////////////////////////////////////
int CUSBio::Load6620Params(int filter)
{
unsigned int tmp = 0;
	if(	(filter!=m_Last6620FilterLoaded) && m_USBPortOpen)	//only if needed
	{
		m_6620params.LoadFixedFilter(filter+1);		//load from filters.h
		Send6620Msg(ADR_MODECTRL, MODECTRL_SREAL|MODECTRL_RESET|MODECTRL_SYNCMASTER);
		if(m_6620params.m_NCOampDither)
			tmp |= NCOCTRL_AMPDITHER;
		if(m_6620params.m_NCOphzDither)
			tmp |= NCOCTRL_PHZDITHER;
		Send6620Msg(ADR_NCOCTRL, tmp);
		Send6620Msg(ADR_CIC2SCALE, m_6620params.m_CIC2Scale);
		Send6620Msg(ADR_CIC2M, m_6620params.m_CIC2Rate-1);
		Send6620Msg(ADR_CIC5SCALE, m_6620params.m_CIC5Scale);
		Send6620Msg(ADR_CIC5M, m_6620params.m_CIC5Rate-1);

//		Send6620Msg(ADR_RCFCTRL, 4);	//this will be loaded when IFGain is set

		Send6620Msg(ADR_RCFM, m_6620params.m_RCFRate-1);
		Send6620Msg(ADR_RCFOFFSET, 0);
		Send6620Msg(ADR_TAPS, m_6620params.m_RCFTaps-1);
		for(int i=0; i<m_6620params.m_RCFTaps; i++)
		{
			Send6620Msg(i, m_6620params.m_FIRcoef[i]);
		}
		Send6620Msg(ADR_MODECTRL, MODECTRL_SREAL|MODECTRL_SYNCMASTER);
		m_Last6620FilterLoaded = filter;
	}
	return m_6620params.m_TotalDecimationRate;
}

/////////////////////////////////////////////////////////////////
//Helper Function that loads a single AD6620 register with data
/////////////////////////////////////////////////////////////////
void CUSBio::Send6620Msg(int adr, unsigned int data)
{
	m_txASCPmsg.words.header = 9;
	m_txASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM1;
	m_txASCPmsg.words.CItem = adr;
	m_txASCPmsg.longs.parms[0] = data;
	m_txASCPmsg.bytes.parms[4] = 0;
	SendASCPMsg( &m_txASCPmsg);
	Sleep(0);		//give other tasks a slice since the load takes a while
}

void CUSBio::ReadIOMemory( int adr, unsigned int* pData)
{
unsigned int val = 0;
	val = (unsigned int)m_EEROMData[adr];
	val = val + ( (unsigned int)m_EEROMData[adr+1] << 8);
	val = val + ( (unsigned int)m_EEROMData[adr+2] << 16);
	val = val + ( (unsigned int)m_EEROMData[adr+3] << 24);
	pData[0] = val;
}

void CUSBio::WriteIOMemory( int adr, unsigned int Data)
{
unsigned int val;
FT_STATUS Status;
	if( m_FTStatus == FT_OK)
	{
		ReadIOMemory( adr, &val);	//read present value
		if(val != Data )	//need to write EERom
		{
			m_EEROMData[adr] = (unsigned char)(Data & 0xFF);
			m_EEROMData[adr+1] = (unsigned char)((Data>>8) & 0xFF);
			m_EEROMData[adr+2] = (unsigned char)((Data>>16) & 0xFF);
			m_EEROMData[adr+3] = (unsigned char)((Data>>24) & 0xFF);
			Status = g_pFT_EE_UAWrite(m_FTHandle, m_EEROMData, EEROM_SIZE);
		}
	}
}

void CUSBio::SetSDRIQSampleRateFromEEROM(unsigned int rate)
{
unsigned char msg[10];
DWORD txbytes;
	if( (rate>50000000) && (rate<70000000) )
	{
		//create ASCP msg
		msg[0] = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
		msg[1] = TYPE_HOST_SET_CITEM;
		msg[2] = (unsigned char)CI_RX_IN_SAMPLE_RATE;
		msg[3] = 0;
		msg[4] = 0;
		msg[5] = (unsigned char)((rate>>0)&0xFF);
		msg[6] = (unsigned char)((rate>>8)&0xFF);
		msg[7] = (unsigned char)((rate>>16)&0xFF);
		msg[8] = (unsigned char)((rate>>24)&0xFF);
		m_FTStatus = g_pWrite( m_FTHandle, (LPVOID)msg, CI_RX_IN_SAMPLE_RATE_SETRESPLEN, &txbytes);
	}
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop of worker thread for removing USB FIFO data
//////////////////////////////////////////////////////////////////////
void CUSBio::FIFOThreadProc(LPVOID pCtrl)
{
DWORD eventstat;
unsigned __int8 data;
	CSDR14XCtrl* pXsdrCtrl = (CSDR14XCtrl*)pCtrl;
	m_FifoHead = 0;
	m_FifoTail = 0;
	AfxGetThread( )->SetThreadPriority(THREAD_PRIORITY_HIGHEST );
	while(pXsdrCtrl->m_ThreadRunning)
	{
		if( m_FifoHead == m_FifoTail )	//if FIFO empty
		{
			eventstat = WaitForSingleObject( m_hFifoWriteEvent, 10);
		}
		while( m_FifoHead != m_FifoTail )	//process until all bytes read from USB Fifo
		{
			data = m_pUSBFifo[m_FifoTail++];
			m_FifoTail &= (USB_FIFOSIZE-1);
			switch(m_MsgState)	//Simple state machine to get generic ASCP msg
			{
				case MSGSTATE_HDR1:		//get first header byte of ASCP msg
					m_rxASCPmsg.bytes.b0 = data;
					m_MsgState = MSGSTATE_HDR2;	//go to get second header byte state
					break;
				case MSGSTATE_HDR2:	//here for second byte of header
					m_rxASCPmsg.bytes.b1 = data;
					m_RxMsgLength = m_rxASCPmsg.words.header&0x001F;
					m_RxMsgIndex = 2;
					if(2 == m_RxMsgLength)	//if msg has no parameters then we are done
					{
						m_MsgState = MSGSTATE_HDR1;	//go back to first state
						pXsdrCtrl->ParseASCPmsg( &m_rxASCPmsg );
					}
					else	//there are data bytes to fetch
					{
						if( 0 == m_RxMsgLength)
							m_RxMsgLength = 8192+2;	//handle special case of 8192 byte data message
						m_MsgState = MSGSTATE_DATA;	//go to data byte reading state
					}
					break;
				case MSGSTATE_DATA:	//try to read the rest of the message or to end of USB buffer
					m_rxASCPmsg.buf[m_RxMsgIndex++] = data;
					if( m_RxMsgIndex >= m_RxMsgLength )
					{	
						m_MsgState = MSGSTATE_HDR1;	//go back to first stage
						pXsdrCtrl->ParseASCPmsg(&m_rxASCPmsg);	//call ASCP message parsing routine
					}

					break;
			} //end switch statement
		} //end while loop
	}
}